"""Access the DHI MIKE21 item and unit objects
"""

import os
import clr
from . import config

# Set path to MIKE SDK
sdk_path = config.MIKE_SDK
eum_dll = config.MIKE_EUM
clr.AddReference(os.path.join(sdk_path, eum_dll))

# Import .NET libraries
from DHI.Generic.MikeZero import eumUnit, eumItem


def get_item(item):
    """
    Return MIKE21 item code. See :func:`available_items()` for all available
    MIKE21 items.

    Parameters
    ----------
    item : str
        Item name from :func:`available_items()`

    Returns
    -------
    item_num : int
        Item number relating to input item
    """
    return mike_items[item]


def get_unit(unit):
    """
    Return MIKE21 unit code. See :func:`available_units()` for all available
    MIKE21 units.

    Parameters
    ----------
    unit : str
        Unit name from :func:`available_units()`

    Returns
    -------
    unit_num : int
        Unit number relating to input unit
    """
    return mike_units[unit]


def available_items():
    """
    Return list of available MIKE21 items

    Returns
    -------
    available_items : list
        Available MIKE21 item names. Get the correct item code when name is
        used in :func:`get_item()`.
    """
    return list(mike_items.keys())


def available_units():
    """
    Return list of available MIKE21 units

    Returns
    -------
    available_units : list
        Available MIKE21 unit names. Get the correct unit code when name is
        used in :func:`get_unit()`.
    """
    return list(mike_units.keys())


mike_items = {
    '2DSurfaceElevationSpectrum': eumItem.eumI2DSurfaceElevationSpectrum,
    '3DSurfaceElevationSpectrum': eumItem.eumI3DSurfaceElevationSpectrum,
    'AbsoluteHumidity': eumItem.eumIAbsoluteHumidity,
    'AccCompConvTransp': eumItem.eumIAccCompConvTransp,
    'AccCompDecay': eumItem.eumIAccCompDecay,
    'AccCompDispTransp': eumItem.eumIAccCompDispTransp,
    'AccCompMassError': eumItem.eumIAccCompMassError,
    'AccCompTransp': eumItem.eumIAccCompTransp,
    'AccSediTransportPerM': eumItem.eumIAccSediTransportPerM,
    'AccSedimentMass': eumItem.eumIAccSedimentMass,
    'AccSedimentTransport': eumItem.eumIAccSedimentTransport,
    'AccWaterVolumeError': eumItem.eumIAccWaterVolumeError,
    'AccumulatedErosion': eumItem.eumIAccumulatedErosion,
    'AcousticAttenuation': eumItem.eumIAcousticAttenuation,
    'ActiveBeachHeight': eumItem.eumIActiveBeachHeight,
    'ActiveDepth': eumItem.eumIActiveDepth,
    'AddedMassCoefficient': eumItem.eumIAddedMassCoefficient,
    'AddedMassRR': eumItem.eumIAddedMassRR,
    'AddedMassRT': eumItem.eumIAddedMassRT,
    'AddedMassTR': eumItem.eumIAddedMassTR,
    'AddedMassTT': eumItem.eumIAddedMassTT,
    'AdsorptionCoefficient': eumItem.eumIAdsorptionCoefficient,
    'Age': eumItem.eumIAge,
    'AgentDensity': eumItem.eumIAgentDensity,
    'AgriculturalArea': eumItem.eumIAgriculturalArea,
    'AgriculturalUsage': eumItem.eumIAgriculturalUsage,
    'AirPressure': eumItem.eumIAirPressure,
    'Angle': eumItem.eumIAngle,
    'Angle2': eumItem.eumIAngle2,
    'AngleBedVelocity': eumItem.eumIAngleBedVelocity,
    'AngleOfRespose': eumItem.eumIAngleOfRespose,
    'ApplicationIntensity': eumItem.eumIApplicationIntensity,
    'AreaFraction': eumItem.eumIAreaFraction,
    'Aroot': eumItem.eumIAroot,
    'AssimilativeCapacity': eumItem.eumIAssimilativeCapacity,
    'Asymmetry': eumItem.eumIAsymmetry,
    'Atiltness': eumItem.eumIAtiltness,
    'Attenuation': eumItem.eumIAttenuation,
    'AutoSpectrum': eumItem.eumIAutoSpectrum,
    'AuxiliaryVariableW': eumItem.eumIAuxiliaryVariableW,
    'AveSediTransportPerLengthUnit': eumItem.eumIAveSediTransportPerLengthUnit,
    'AverageLength': eumItem.eumIAverageLength,
    'BODAtRiverBed': eumItem.eumIBODAtRiverBed,
    'BacteriaConc': eumItem.eumIBacteriaConc,
    'BacteriaFlow': eumItem.eumIBacteriaFlow,
    'BacteriaFlux': eumItem.eumIBacteriaFlux,
    'Bathymetry': eumItem.eumIBathymetry,
    'BeachPosition': eumItem.eumIBeachPosition,
    'BedDistribution': eumItem.eumIBedDistribution,
    'BedLevel': eumItem.eumIBedLevel,
    'BedLevelChange': eumItem.eumIBedLevelChange,
    'BedLoadPerM': eumItem.eumIBedLoadPerM,
    'BedMass': eumItem.eumIBedMass,
    'BedSedimentLoad': eumItem.eumIBedSedimentLoad,
    'BedSlope': eumItem.eumIBedSlope,
    'BedThickness': eumItem.eumIBedThickness,
    'BioconcentrationFactor': eumItem.eumIBioconcentrationFactor,
    'BottomLevel': eumItem.eumIBottomLevel,
    'BottomLevelChange': eumItem.eumIBottomLevelChange,
    'BoundaryLayerThickness': eumItem.eumIBoundaryLayerThickness,
    'BreakingParameterGamma': eumItem.eumIBreakingParameterGamma,
    'BreakingWave': eumItem.eumIBreakingWave,
    'C1': eumItem.eumIC1,
    'C2': eumItem.eumIC2,
    'C3': eumItem.eumIC3,
    'CapillaryFlux': eumItem.eumICapillaryFlux,
    'CatchmentArea': eumItem.eumICatchmentArea,
    'CatchmentSlope': eumItem.eumICatchmentSlope,
    'CellHeight': eumItem.eumICellHeight,
    'CharnockConstant': eumItem.eumICharnockConstant,
    'ChezyNo': eumItem.eumIChezyNo,
    'ClearnessCoefficient': eumItem.eumIClearnessCoefficient,
    'ClimateNumber': eumItem.eumIClimateNumber,
    'Cloudiness': eumItem.eumICloudiness,
    'CoSpectrum': eumItem.eumICoSpectrum,
    'CoherenceSpectrum': eumItem.eumICoherenceSpectrum,
    'CoherentSpectrum': eumItem.eumICoherentSpectrum,
    'CompConvTransp': eumItem.eumICompConvTransp,
    'CompDecay': eumItem.eumICompDecay,
    'CompDispTransp': eumItem.eumICompDispTransp,
    'CompMass': eumItem.eumICompMass,
    'CompMassError': eumItem.eumICompMassError,
    'CompTransp': eumItem.eumICompTransp,
    'Concentration': eumItem.eumIConcentration,
    'Concentration1': eumItem.eumIConcentration1,
    'Concentration_1': eumItem.eumIConcentration_1,
    'Concentration_2': eumItem.eumIConcentration_2,
    'Concentration_3': eumItem.eumIConcentration_3,
    'Concentration_4': eumItem.eumIConcentration_4,
    'ConstFallVelocity': eumItem.eumIConstFallVelocity,
    'ContourAngle': eumItem.eumIContourAngle,
    'Conveyance': eumItem.eumIConveyance,
    'ConveyanceLoss': eumItem.eumIConveyanceLoss,
    'ConveyanceReduction': eumItem.eumIConveyanceReduction,
    'CorrectionOfPrecipitation': eumItem.eumICorrectionOfPrecipitation,
    'Cost': eumItem.eumICost,
    'CostPerEnergy': eumItem.eumICostPerEnergy,
    'CostPerMass': eumItem.eumICostPerMass,
    'CostPerTime': eumItem.eumICostPerTime,
    'CostPerVolume': eumItem.eumICostPerVolume,
    'CountsPerLiter': eumItem.eumICountsPerLiter,
    'CourantNumber': eumItem.eumICourantNumber,
    'CriticalVelocity': eumItem.eumICriticalVelocity,
    'CropDemand': eumItem.eumICropDemand,
    'CropIndex': eumItem.eumICropIndex,
    'CropLoss': eumItem.eumICropLoss,
    'CropStage': eumItem.eumICropStage,
    'CropStress': eumItem.eumICropStress,
    'CropType': eumItem.eumICropType,
    'CropYield': eumItem.eumICropYield,
    'CrossSectionXdata': eumItem.eumICrossSectionXdata,
    'CrossSpectrum': eumItem.eumICrossSpectrum,
    'CurrentDirection': eumItem.eumICurrentDirection,
    'CurrentMagnitude': eumItem.eumICurrentMagnitude,
    'CurrentSpeed': eumItem.eumICurrentSpeed,
    'DSD': eumItem.eumIDSD,
    'DamBreachLevel': eumItem.eumIDamBreachLevel,
    'DamBreachSlope': eumItem.eumIDamBreachSlope,
    'DamBreachWidth': eumItem.eumIDamBreachWidth,
    'Damcrestlevel': eumItem.eumIDamcrestlevel,
    'DampingRR': eumItem.eumIDampingRR,
    'DampingRT': eumItem.eumIDampingRT,
    'DampingTR': eumItem.eumIDampingTR,
    'DampingTT': eumItem.eumIDampingTT,
    'DarcyVelocity': eumItem.eumIDarcyVelocity,
    'DecayFactor': eumItem.eumIDecayFactor,
    'Deficit': eumItem.eumIDeficit,
    'Deflection': eumItem.eumIDeflection,
    'Demandcarryoverfraction': eumItem.eumIDemandcarryoverfraction,
    'Density': eumItem.eumIDensity,
    'DepositionConcFlux': eumItem.eumIDepositionConcFlux,
    'DepositionRate': eumItem.eumIDepositionRate,
    'DepthBelowGround': eumItem.eumIDepthBelowGround,
    'DepthofWind': eumItem.eumIDepthofWind,
    'DesorptionCoefficient': eumItem.eumIDesorptionCoefficient,
    'DiffusionCoefficient': eumItem.eumIDiffusionCoefficient,
    'DirStdDev': eumItem.eumIDirStdDev,
    'DirecIntegSpectralDensity': eumItem.eumIDirecIntegSpectralDensity,
    'DirectionalSpreading': eumItem.eumIDirectionalSpreading,
    'DirectionalSpreadingFunction': eumItem.eumIDirectionalSpreadingFunction,
    'Discharge': eumItem.eumIDischarge,
    'DischargeAmplitudeError': eumItem.eumIDischargeAmplitudeError,
    'DischargeCorrection': eumItem.eumIDischargeCorrection,
    'DischargePerTime': eumItem.eumIDischargePerTime,
    'DischargeSimulated': eumItem.eumIDischargeSimulated,
    'DispCoef': eumItem.eumIDispCoef,
    'DispFact': eumItem.eumIDispFact,
    'DispersantActivity': eumItem.eumIDispersantActivity,
    'DispersionCoefficient': eumItem.eumIDispersionCoefficient,
    'DispersionVelocityFactor': eumItem.eumIDispersionVelocityFactor,
    'Dispersivity': eumItem.eumIDispersivity,
    'DissipationTKE': eumItem.eumIDissipationTKE,
    'Distance': eumItem.eumIDistance,
    'Diverteddischarge': eumItem.eumIDiverteddischarge,
    'Drainage': eumItem.eumIDrainage,
    'DrainageTimeConstant': eumItem.eumIDrainageTimeConstant,
    'DredgeRate': eumItem.eumIDredgeRate,
    'DredgeSpill': eumItem.eumIDredgeSpill,
    'DuneHeight': eumItem.eumIDuneHeight,
    'DuneLength': eumItem.eumIDuneLength,
    'DunePosition': eumItem.eumIDunePosition,
    'Duration': eumItem.eumIDuration,
    'DynamicViscosity': eumItem.eumIDynamicViscosity,
    'EPANETFriction': eumItem.eumIEPANETFriction,
    'EPANETReactionRate': eumItem.eumIEPANETReactionRate,
    'EPANETSetting': eumItem.eumIEPANETSetting,
    'EPANETStatus': eumItem.eumIEPANETStatus,
    'EPANETWaterQuality': eumItem.eumIEPANETWaterQuality,
    'ETRate': eumItem.eumIETRate,
    'ElasticStorage': eumItem.eumIElasticStorage,
    'ElectricConductivity': eumItem.eumIElectricConductivity,
    'ElectricCurrent': eumItem.eumIElectricCurrent,
    'ElementArea': eumItem.eumIElementArea,
    'ElementAreaLongLat': eumItem.eumIElementAreaLongLat,
    'ElementLength': eumItem.eumIElementLength,
    'ElementVolume': eumItem.eumIElementVolume,
    'Elevation': eumItem.eumIElevation,
    'EllipsoidalHeight': eumItem.eumIEllipsoidalHeight,
    'EmitterCoefficient': eumItem.eumIEmitterCoefficient,
    'EmulsificationConstantK1': eumItem.eumIEmulsificationConstantK1,
    'EmulsificationConstantK2': eumItem.eumIEmulsificationConstantK2,
    'EncroachmentOffset': eumItem.eumIEncroachmentOffset,
    'EncroachmentPosition': eumItem.eumIEncroachmentPosition,
    'EncroachmentWidth': eumItem.eumIEncroachmentWidth,
    'EnergyLevelChange': eumItem.eumIEnergyLevelChange,
    'EpsilonUZ': eumItem.eumIEpsilonUZ,
    'EroDepoCoef': eumItem.eumIEroDepoCoef,
    'ErosionCoefficient': eumItem.eumIErosionCoefficient,
    'ErosionCoefficientFactor': eumItem.eumIErosionCoefficientFactor,
    'ErosionConcentration': eumItem.eumIErosionConcentration,
    'ErosionRate': eumItem.eumIErosionRate,
    'ErosionSurfaceLoad': eumItem.eumIErosionSurfaceLoad,
    'Evaporation': eumItem.eumIEvaporation,
    'EvaporationFlux': eumItem.eumIEvaporationFlux,
    'EvaporationIntesity': eumItem.eumIEvaporationIntesity,
    'EvaporationRate': eumItem.eumIEvaporationRate,
    'Evaporationfraction': eumItem.eumIEvaporationfraction,
    'ExchangeRate': eumItem.eumIExchangeRate,
    'FIRCoefficient': eumItem.eumIFIRCoefficient,
    'FRDischarge': eumItem.eumIFRDischarge,
    'FallVelocity': eumItem.eumIFallVelocity,
    'FcoliConcentration': eumItem.eumIFcoliConcentration,
    'FenderForce': eumItem.eumIFenderForce,
    'FetchLength': eumItem.eumIFetchLength,
    'FilterCoefficient': eumItem.eumIFilterCoefficient,
    'FirstMomentWaveAction': eumItem.eumIFirstMomentWaveAction,
    'FirstOrderGrazingRateDependance': eumItem.eumIFirstOrderGrazingRateDependance,
    'FlapPosition': eumItem.eumIFlapPosition,
    'FloodedArea': eumItem.eumIFloodedArea,
    'FlowArea': eumItem.eumIFlowArea,
    'FlowDirection': eumItem.eumIFlowDirection,
    'FlowDirection_1': eumItem.eumIFlowDirection_1,
    'FlowFlux': eumItem.eumIFlowFlux,
    'FlowRoutingTimeCnst': eumItem.eumIFlowRoutingTimeCnst,
    'FlowVelocity': eumItem.eumIFlowVelocity,
    'FlowVelocityProfile': eumItem.eumIFlowVelocityProfile,
    'FlowWidth': eumItem.eumIFlowWidth,
    'Force': eumItem.eumIForce,
    'ForceLoggedData': eumItem.eumIForceLoggedData,
    'FourierACoefficient': eumItem.eumIFourierACoefficient,
    'FourierBCoefficient': eumItem.eumIFourierBCoefficient,
    'Fraction': eumItem.eumIFraction,
    'FrameRate': eumItem.eumIFrameRate,
    'FreqIntegSpectralDensity': eumItem.eumIFreqIntegSpectralDensity,
    'Frequency': eumItem.eumIFrequency,
    'FrequencyResponseSpectrum': eumItem.eumIFrequencyResponseSpectrum,
    'FrictionCoeffient': eumItem.eumIFrictionCoeffient,
    'FrictionCoeffientcfw': eumItem.eumIFrictionCoeffientcfw,
    'FrictionFactor': eumItem.eumIFrictionFactor,
    'FroudeNo': eumItem.eumIFroudeNo,
    'FstOrderRateAD': eumItem.eumIFstOrderRateAD,
    'FstOrderRateWQ': eumItem.eumIFstOrderRateWQ,
    'GateLevel': eumItem.eumIGateLevel,
    'GeoDeviation': eumItem.eumIGeoDeviation,
    'GeographicalCoordinate': eumItem.eumIGeographicalCoordinate,
    'GrainDiameter': eumItem.eumIGrainDiameter,
    'Gravity': eumItem.eumIGravity,
    'GridRotation': eumItem.eumIGridRotation,
    'GridSpacing': eumItem.eumIGridSpacing,
    'GroundWaterAbstraction': eumItem.eumIGroundWaterAbstraction,
    'GroundWaterAbstractionDepth': eumItem.eumIGroundWaterAbstractionDepth,
    'GroundWaterAbstractionFlux': eumItem.eumIGroundWaterAbstractionFlux,
    'GroundWaterDepth': eumItem.eumIGroundWaterDepth,
    'GroundWaterFlowVelocity': eumItem.eumIGroundWaterFlowVelocity,
    'GroundwaterRecharge': eumItem.eumIGroundwaterRecharge,
    'Groundwaterdemand': eumItem.eumIGroundwaterdemand,
    'HabitatIndex': eumItem.eumIHabitatIndex,
    'HalfLife': eumItem.eumIHalfLife,
    'HalfOrderRateWQ': eumItem.eumIHalfOrderRateWQ,
    'HeadDifference': eumItem.eumIHeadDifference,
    'HeadElevation': eumItem.eumIHeadElevation,
    'HeadGradient': eumItem.eumIHeadGradient,
    'HeatFlux': eumItem.eumIHeatFlux,
    'HeatFluxResistance': eumItem.eumIHeatFluxResistance,
    'HeightAboveGround': eumItem.eumIHeightAboveGround,
    'HeightAboveKeel': eumItem.eumIHeightAboveKeel,
    'HydrConductance': eumItem.eumIHydrConductance,
    'HydrConductivity': eumItem.eumIHydrConductivity,
    'HydrLeakageCoefficient': eumItem.eumIHydrLeakageCoefficient,
    'HydrTransmissivity': eumItem.eumIHydrTransmissivity,
    'HydraulicLength': eumItem.eumIHydraulicLength,
    'HydraulicRadius': eumItem.eumIHydraulicRadius,
    'HydrostaticMatrix': eumItem.eumIHydrostaticMatrix,
    'IF1_Flow': eumItem.eumIIF1_Flow,
    'IceConcentration': eumItem.eumIIceConcentration,
    'IceThickness': eumItem.eumIIceThickness,
    'IncidentSpectrum': eumItem.eumIIncidentSpectrum,
    'IndustrialOutput': eumItem.eumIIndustrialOutput,
    'IndustrialUse': eumItem.eumIIndustrialUse,
    'Infiltration': eumItem.eumIInfiltration,
    'InfiltrationFlux': eumItem.eumIInfiltrationFlux,
    'IntegerCode': eumItem.eumIIntegerCode,
    'Interception': eumItem.eumIInterception,
    'InverseExpo': eumItem.eumIInverseExpo,
    'IrrigatedArea': eumItem.eumIIrrigatedArea,
    'Irrigation': eumItem.eumIIrrigation,
    'IrrigationDemand': eumItem.eumIIrrigationDemand,
    'IrrigationIndex': eumItem.eumIIrrigationIndex,
    'IrrigationRate': eumItem.eumIIrrigationRate,
    'ItemGeometry0D': eumItem.eumIItemGeometry0D,
    'ItemGeometry1D': eumItem.eumIItemGeometry1D,
    'ItemGeometry2D': eumItem.eumIItemGeometry2D,
    'ItemGeometry3D': eumItem.eumIItemGeometry3D,
    'ItemUndefined': eumItem.eumIItemUndefined,
    'KC': eumItem.eumIKC,
    'Kd': eumItem.eumIKd,
    'Kurtosis': eumItem.eumIKurtosis,
    'LAI': eumItem.eumILAI,
    'LatLong': eumItem.eumILatLong,
    'LatentHeat': eumItem.eumILatentHeat,
    'LayerThickness': eumItem.eumILayerThickness,
    'Leakage': eumItem.eumILeakage,
    'Length': eumItem.eumILength,
    'LengthError': eumItem.eumILengthError,
    'LengthLoggedData': eumItem.eumILengthLoggedData,
    'LengthScale': eumItem.eumILengthScale,
    'LengthZeroCrossing': eumItem.eumILengthZeroCrossing,
    'LevelAmplitudeError': eumItem.eumILevelAmplitudeError,
    'LevelCorrection': eumItem.eumILevelCorrection,
    'LevelSimulated': eumItem.eumILevelSimulated,
    'LightExtinction': eumItem.eumILightExtinction,
    'LightExtinctionBackground': eumItem.eumILightExtinctionBackground,
    'LineCapacity': eumItem.eumILineCapacity,
    'LineThickness': eumItem.eumILineThickness,
    'LinearDampingCoefficient': eumItem.eumILinearDampingCoefficient,
    'LinearRoutingCoef': eumItem.eumILinearRoutingCoef,
    'LiveStockDemand': eumItem.eumILiveStockDemand,
    'LocalCoordinate': eumItem.eumILocalCoordinate,
    'Logical': eumItem.eumILogical,
    'LongitudinalInfiltration': eumItem.eumILongitudinalInfiltration,
    'LowerBaseflow': eumItem.eumILowerBaseflow,
    'MachineEfficiency': eumItem.eumIMachineEfficiency,
    'ManningsM': eumItem.eumIManningsM,
    'Manningsn': eumItem.eumIManningsn,
    'MarkerSize': eumItem.eumIMarkerSize,
    'Mass': eumItem.eumIMass,
    'MassFlux': eumItem.eumIMassFlux,
    'MassPerAreaPerTime': eumItem.eumIMassPerAreaPerTime,
    'MassPerTime': eumItem.eumIMassPerTime,
    'MassPerUnitArea': eumItem.eumIMassPerUnitArea,
    'MaxWater': eumItem.eumIMaxWater,
    'MaxWaveHeight': eumItem.eumIMaxWaveHeight,
    'MeanWaveDirection': eumItem.eumIMeanWaveDirection,
    'MeltingCoefficient': eumItem.eumIMeltingCoefficient,
    'Moment': eumItem.eumIMoment,
    'MomentOfInertia': eumItem.eumIMomentOfInertia,
    'NonDimExp': eumItem.eumINonDimExp,
    'NonDimFactor': eumItem.eumINonDimFactor,
    'NonDimTransport': eumItem.eumINonDimTransport,
    'NormalVelocity': eumItem.eumINormalVelocity,
    'NumberOfLiveStock': eumItem.eumINumberOfLiveStock,
    'NumberOfParticles': eumItem.eumINumberOfParticles,
    'OF1_Flow': eumItem.eumIOF1_Flow,
    'Ownedfractionofinflow': eumItem.eumIOwnedfractionofinflow,
    'Ownedfractionofvolume': eumItem.eumIOwnedfractionofvolume,
    'ParticleProductionRate': eumItem.eumIParticleProductionRate,
    'ParticleVelocityU': eumItem.eumIParticleVelocityU,
    'ParticleVelocityV': eumItem.eumIParticleVelocityV,
    'Percentage': eumItem.eumIPercentage,
    'PersonEqui': eumItem.eumIPersonEqui,
    'PhaseError': eumItem.eumIPhaseError,
    'PhaseSpectrum': eumItem.eumIPhaseSpectrum,
    'PipeDiameter': eumItem.eumIPipeDiameter,
    'PistonPosition': eumItem.eumIPistonPosition,
    'PollutantLoad': eumItem.eumIPollutantLoad,
    'Population': eumItem.eumIPopulation,
    'PopulationUsage': eumItem.eumIPopulationUsage,
    'Porosity': eumItem.eumIPorosity,
    'PorosityCoefficient': eumItem.eumIPorosityCoefficient,
    'PotentialEvapRate': eumItem.eumIPotentialEvapRate,
    'Power': eumItem.eumIPower,
    'Precipitation': eumItem.eumIPrecipitation,
    'PrecipitationCorrection': eumItem.eumIPrecipitationCorrection,
    'PrecipitationRate': eumItem.eumIPrecipitationRate,
    'Pressure': eumItem.eumIPressure,
    'PressureHead': eumItem.eumIPressureHead,
    'PressureSI': eumItem.eumIPressureSI,
    'PrintScaleEquivalence': eumItem.eumIPrintScaleEquivalence,
    'Probability': eumItem.eumIProbability,
    'ProfileNumber': eumItem.eumIProfileNumber,
    'ProfileOrientation': eumItem.eumIProfileOrientation,
    'PropagationDirection': eumItem.eumIPropagationDirection,
    'PumpingRate': eumItem.eumIPumpingRate,
    'QuadSpectrum': eumItem.eumIQuadSpectrum,
    'QuadraticDampingCoefficient': eumItem.eumIQuadraticDampingCoefficient,
    'RAOmotion': eumItem.eumIRAOmotion,
    'RAOrotation': eumItem.eumIRAOrotation,
    'RadiationDamping': eumItem.eumIRadiationDamping,
    'RadiationIntensity': eumItem.eumIRadiationIntensity,
    'RadiationMeltingCoefficient': eumItem.eumIRadiationMeltingCoefficient,
    'RadiationPotential': eumItem.eumIRadiationPotential,
    'RadiationStress': eumItem.eumIRadiationStress,
    'RadiiOfGyration': eumItem.eumIRadiiOfGyration,
    'RainMeltingCoefficient': eumItem.eumIRainMeltingCoefficient,
    'RainMeltingCoefficientPerDegree': eumItem.eumIRainMeltingCoefficientPerDegree,
    'Rainfall': eumItem.eumIRainfall,
    'RainfallDepth': eumItem.eumIRainfallDepth,
    'RainfallIntensity': eumItem.eumIRainfallIntensity,
    'RainfallRate': eumItem.eumIRainfallRate,
    'RateBedLevelChange': eumItem.eumIRateBedLevelChange,
    'RearationConstant': eumItem.eumIRearationConstant,
    'Recharge': eumItem.eumIRecharge,
    'ReducedPollutantLoad': eumItem.eumIReducedPollutantLoad,
    'ReductionFactor': eumItem.eumIReductionFactor,
    'Reductionfraction': eumItem.eumIReductionfraction,
    'Reductionlevel': eumItem.eumIReductionlevel,
    'Reductionthreshold': eumItem.eumIReductionthreshold,
    'RefPointNumber': eumItem.eumIRefPointNumber,
    'ReferenceSettlingVelocity': eumItem.eumIReferenceSettlingVelocity,
    'ReflectedSpectrum': eumItem.eumIReflectedSpectrum,
    'ReflectionCoefficient': eumItem.eumIReflectionCoefficient,
    'ReflectionFunction': eumItem.eumIReflectionFunction,
    'RelAccCompMassError': eumItem.eumIRelAccCompMassError,
    'RelCompMassError': eumItem.eumIRelCompMassError,
    'RelMoistureCont': eumItem.eumIRelMoistureCont,
    'RelativeHumidity': eumItem.eumIRelativeHumidity,
    'Residencetime': eumItem.eumIResidencetime,
    'ResistFactor': eumItem.eumIResistFactor,
    'ResistanceRadius': eumItem.eumIResistanceRadius,
    'RespProdPerArea': eumItem.eumIRespProdPerArea,
    'RespProdPerVolume': eumItem.eumIRespProdPerVolume,
    'ResuspensionRate': eumItem.eumIResuspensionRate,
    'ReturnFlowFraction': eumItem.eumIReturnFlowFraction,
    'ReturnPeriod': eumItem.eumIReturnPeriod,
    'RillDepth': eumItem.eumIRillDepth,
    'RiverChainage': eumItem.eumIRiverChainage,
    'RiverStructGeo': eumItem.eumIRiverStructGeo,
    'RiverWidth': eumItem.eumIRiverWidth,
    'RollerAngle': eumItem.eumIRollerAngle,
    'RollerCelerity': eumItem.eumIRollerCelerity,
    'RollerThickness': eumItem.eumIRollerThickness,
    'RootDepth': eumItem.eumIRootDepth,
    'Roughness': eumItem.eumIRoughness,
    'RoughnessLength': eumItem.eumIRoughnessLength,
    'RubbleMound': eumItem.eumIRubbleMound,
    'SCSCatchSlope': eumItem.eumISCSCatchSlope,
    'SRDischarge': eumItem.eumISRDischarge,
    'Salinity': eumItem.eumISalinity,
    'SaltFlux': eumItem.eumISaltFlux,
    'ScourDepth': eumItem.eumIScourDepth,
    'ScourWidth': eumItem.eumIScourWidth,
    'SediTransportPerM': eumItem.eumISediTransportPerM,
    'SedimentBedDensity': eumItem.eumISedimentBedDensity,
    'SedimentDepth': eumItem.eumISedimentDepth,
    'SedimentDiameter': eumItem.eumISedimentDiameter,
    'SedimentFraction': eumItem.eumISedimentFraction,
    'SedimentFractionChange': eumItem.eumISedimentFractionChange,
    'SedimentGradation': eumItem.eumISedimentGradation,
    'SedimentGrainDiameter': eumItem.eumISedimentGrainDiameter,
    'SedimentLayer': eumItem.eumISedimentLayer,
    'SedimentMassPerM': eumItem.eumISedimentMassPerM,
    'SedimentSpill': eumItem.eumISedimentSpill,
    'SedimentTransport': eumItem.eumISedimentTransport,
    'SedimentTransportDirection': eumItem.eumISedimentTransportDirection,
    'SedimentVolumePerLengthUnit': eumItem.eumISedimentVolumePerLengthUnit,
    'SedimentationVelocity': eumItem.eumISedimentationVelocity,
    'Seepage': eumItem.eumISeepage,
    'Seepageflux': eumItem.eumISeepageflux,
    'Seepagefraction': eumItem.eumISeepagefraction,
    'SettlingVelocityCoef': eumItem.eumISettlingVelocityCoef,
    'ShearStress': eumItem.eumIShearStress,
    'ShieldsParameter': eumItem.eumIShieldsParameter,
    'ShoreCurrentZone': eumItem.eumIShoreCurrentZone,
    'SignificantWaveHeight': eumItem.eumISignificantWaveHeight,
    'SizeAndPosition': eumItem.eumISizeAndPosition,
    'Skewness': eumItem.eumISkewness,
    'SnowCover': eumItem.eumISnowCover,
    'SnowCoverPercentage': eumItem.eumISnowCoverPercentage,
    'SnowDepth': eumItem.eumISnowDepth,
    'SoluteFlux': eumItem.eumISoluteFlux,
    'SoundExposureLevel': eumItem.eumISoundExposureLevel,
    'SoundSpeed': eumItem.eumISoundSpeed,
    'SpecificDischarge': eumItem.eumISpecificDischarge,
    'SpecificGravity': eumItem.eumISpecificGravity,
    'SpecificHeat': eumItem.eumISpecificHeat,
    'SpecificPrecipitation': eumItem.eumISpecificPrecipitation,
    'SpecificRunoff': eumItem.eumISpecificRunoff,
    'SpecificSoluteFluxPerArea': eumItem.eumISpecificSoluteFluxPerArea,
    'SpecificYield': eumItem.eumISpecificYield,
    'SpectralDensity': eumItem.eumISpectralDensity,
    'SpectralDescription': eumItem.eumISpectralDescription,
    'SpeedLoggedData': eumItem.eumISpeedLoggedData,
    'SpongeCoefficient': eumItem.eumISpongeCoefficient,
    'SpreadingFactor': eumItem.eumISpreadingFactor,
    'StillWaterDepth': eumItem.eumIStillWaterDepth,
    'StorageChangeRate': eumItem.eumIStorageChangeRate,
    'StorageDepth': eumItem.eumIStorageDepth,
    'StructureGeometryPerTime': eumItem.eumIStructureGeometryPerTime,
    'SubFlapPosition': eumItem.eumISubFlapPosition,
    'SubPistonPosition': eumItem.eumISubPistonPosition,
    'SubmergedMass': eumItem.eumISubmergedMass,
    'SummQCorrected': eumItem.eumISummQCorrected,
    'SunRadiation': eumItem.eumISunRadiation,
    'SunShine': eumItem.eumISunShine,
    'SupFlapPosition': eumItem.eumISupFlapPosition,
    'SupPistonPosition': eumItem.eumISupPistonPosition,
    'SurfStorage_OF0': eumItem.eumISurfStorage_OF0,
    'SurfStorage_OF1': eumItem.eumISurfStorage_OF1,
    'SurfaceArea': eumItem.eumISurfaceArea,
    'SurfaceCurvature': eumItem.eumISurfaceCurvature,
    'SurfaceElevation': eumItem.eumISurfaceElevation,
    'SurfaceElevationAtPaddle': eumItem.eumISurfaceElevationAtPaddle,
    'SurfaceSlope': eumItem.eumISurfaceSlope,
    'SuspLoadPerM': eumItem.eumISuspLoadPerM,
    'SuspSedimentLoad': eumItem.eumISuspSedimentLoad,
    'TargetPower': eumItem.eumITargetPower,
    'Temperature': eumItem.eumITemperature,
    'TemperatureCorrection': eumItem.eumITemperatureCorrection,
    'TemperatureFlux': eumItem.eumITemperatureFlux,
    'TemperatureLapseRate': eumItem.eumITemperatureLapseRate,
    'ThresholdPeriod': eumItem.eumIThresholdPeriod,
    'TimeCorrectionAtNoon': eumItem.eumITimeCorrectionAtNoon,
    'TimeFirstWaveArrival': eumItem.eumITimeFirstWaveArrival,
    'TimeScale': eumItem.eumITimeScale,
    'TimeShift': eumItem.eumITimeShift,
    'TimeStep': eumItem.eumITimeStep,
    'TimeStepFactor': eumItem.eumITimeStepFactor,
    'TimeZeroCrossing': eumItem.eumITimeZeroCrossing,
    'Topography': eumItem.eumITopography,
    'TotalGas': eumItem.eumITotalGas,
    'TotalLosses': eumItem.eumITotalLosses,
    'TotalWaterDepth': eumItem.eumITotalWaterDepth,
    'TransferRate': eumItem.eumITransferRate,
    'TransitionRate': eumItem.eumITransitionRate,
    'TransmissionCoefficient': eumItem.eumITransmissionCoefficient,
    'TransmissionLoss': eumItem.eumITransmissionLoss,
    'TrenchPosition': eumItem.eumITrenchPosition,
    'Turbidity_FTU': eumItem.eumITurbidity_FTU,
    'Turbidity_MgPerL': eumItem.eumITurbidity_MgPerL,
    'TurbulentKineticEnergy': eumItem.eumITurbulentKineticEnergy,
    'UnitHydrographOrdinate': eumItem.eumIUnitHydrographOrdinate,
    'UpdatePeriod': eumItem.eumIUpdatePeriod,
    'ValveSetting': eumItem.eumIValveSetting,
    'VegetationHeight': eumItem.eumIVegetationHeight,
    'VesselDisplacement': eumItem.eumIVesselDisplacement,
    'Viscosity': eumItem.eumIViscosity,
    'Visibility': eumItem.eumIVisibility,
    'Voltage': eumItem.eumIVoltage,
    'Volume': eumItem.eumIVolume,
    'VolumeFlowLoggedData': eumItem.eumIVolumeFlowLoggedData,
    'VolumeFlux': eumItem.eumIVolumeFlux,
    'VolumetricWaterContent': eumItem.eumIVolumetricWaterContent,
    'WaterDemand': eumItem.eumIWaterDemand,
    'WaterDepth': eumItem.eumIWaterDepth,
    'WaterLevel': eumItem.eumIWaterLevel,
    'WaterLevelChange': eumItem.eumIWaterLevelChange,
    'WaterVolume': eumItem.eumIWaterVolume,
    'WaterVolumeError': eumItem.eumIWaterVolumeError,
    'WaveActionDensity': eumItem.eumIWaveActionDensity,
    'WaveActionDensityRate': eumItem.eumIWaveActionDensityRate,
    'WaveDirection': eumItem.eumIWaveDirection,
    'WaveDisturbanceCoefficient': eumItem.eumIWaveDisturbanceCoefficient,
    'WaveEnergy': eumItem.eumIWaveEnergy,
    'WaveEnergyDensity': eumItem.eumIWaveEnergyDensity,
    'WaveEnergyDistribution': eumItem.eumIWaveEnergyDistribution,
    'WaveFrequency': eumItem.eumIWaveFrequency,
    'WaveHeight': eumItem.eumIWaveHeight,
    'WaveNumber': eumItem.eumIWaveNumber,
    'WavePeriod': eumItem.eumIWavePeriod,
    'WavePower': eumItem.eumIWavePower,
    'Waveamplitude': eumItem.eumIWaveamplitude,
    'WindDirection': eumItem.eumIWindDirection,
    'WindDragCoefficient': eumItem.eumIWindDragCoefficient,
    'WindFrictionFactor': eumItem.eumIWindFrictionFactor,
    'WindFrictionSpeed': eumItem.eumIWindFrictionSpeed,
    'WindSpeed': eumItem.eumIWindSpeed,
    'WindVelocity': eumItem.eumIWindVelocity,
    'Yieldfactor': eumItem.eumIYieldfactor,
    'ZeroMomentWaveAction': eumItem.eumIZeroMomentWaveAction,
    'energy': eumItem.eumIenergy,
    'pH': eumItem.eumIpH,
    'primaryProduction': eumItem.eumIprimaryProduction,
    'prodPerVolume': eumItem.eumIprodPerVolume,
    'productionRate': eumItem.eumIproductionRate,
    'secchiDepth': eumItem.eumIsecchiDepth,
    'sedimentMass': eumItem.eumIsedimentMass,
    'uVelocity': eumItem.eumIuVelocity,
    'vVelocity': eumItem.eumIvVelocity,
    'wVelocity': eumItem.eumIwVelocity
}

mike_units = {
    'AcreFeetPerDayPerAcre': eumUnit.eumUAcreFeetPerDayPerAcre,
    'AcreFeetPerDayPerSecond': eumUnit.eumUAcreFeetPerDayPerSecond,
    'Bar': eumUnit.eumUBar,
    'CentiMeterPerSecond': eumUnit.eumUCentiMeterPerSecond,
    'Centipoise': eumUnit.eumUCentipoise,
    'ConcNonDimM3PerSec': eumUnit.eumUConcNonDimM3PerSec,
    'CubicFeetPerSecondPerAcre': eumUnit.eumUCubicFeetPerSecondPerAcre,
    'CubicMeterPerDayPerHectar': eumUnit.eumUCubicMeterPerDayPerHectar,
    'CubicMeterPerDayPerSecond': eumUnit.eumUCubicMeterPerDayPerSecond,
    'CubicMeterPerHourPerHectar': eumUnit.eumUCubicMeterPerHourPerHectar,
    'CubicMeterPerHourPerSecond': eumUnit.eumUCubicMeterPerHourPerSecond,
    'CubicMeterPerSecondPerHectar': eumUnit.eumUCubicMeterPerSecondPerHectar,
    'CurrencyPerCubicFeet': eumUnit.eumUCurrencyPerCubicFeet,
    'CurrencyPerCubicMeter': eumUnit.eumUCurrencyPerCubicMeter,
    'DegreeCelsiusM3PerSec': eumUnit.eumUDegreeCelsiusM3PerSec,
    'DegreeCelsiusPer100feet': eumUnit.eumUDegreeCelsiusPer100feet,
    'DegreeCelsiusPer100meter': eumUnit.eumUDegreeCelsiusPer100meter,
    'DegreeFahrenheitFt3PerSec': eumUnit.eumUDegreeFahrenheitFt3PerSec,
    'DegreeFahrenheitPer100feet': eumUnit.eumUDegreeFahrenheitPer100feet,
    'DegreeFahrenheitPer100meter': eumUnit.eumUDegreeFahrenheitPer100meter,
    'DegreeNorth50': eumUnit.eumUDegreeNorth50,
    'EPerM2PerDay': eumUnit.eumUEPerM2PerDay,
    'Feet2One3rdPerSec': eumUnit.eumUFeet2One3rdPerSec,
    'Feet2OneHalfPerSec': eumUnit.eumUFeet2OneHalfPerSec,
    'FeetOfWater': eumUnit.eumUFeetOfWater,
    'FeetPerSecondPerAcre': eumUnit.eumUFeetPerSecondPerAcre,
    'FeetPerSecondPerSecond': eumUnit.eumUFeetPerSecondPerSecond,
    'FeetUS2OneHalfPerSec': eumUnit.eumUFeetUS2OneHalfPerSec,
    'GalPerDay': eumUnit.eumUGalPerDay,
    'GalPerDayPerFeet': eumUnit.eumUGalPerDayPerFeet,
    'GalPerDayPerHead': eumUnit.eumUGalPerDayPerHead,
    'GalPerMin': eumUnit.eumUGalPerMin,
    'GalPerMinutePerFeet': eumUnit.eumUGalPerMinutePerFeet,
    'GalPerMinutePerSecond': eumUnit.eumUGalPerMinutePerSecond,
    'GalPerSec': eumUnit.eumUGalPerSec,
    'GalPerYear': eumUnit.eumUGalPerYear,
    'GalUKPerDayPerFeet': eumUnit.eumUGalUKPerDayPerFeet,
    'GalUKPerDayPerHead': eumUnit.eumUGalUKPerDayPerHead,
    'GalUKPerSec': eumUnit.eumUGalUKPerSec,
    'GalUKPerYear': eumUnit.eumUGalUKPerYear,
    'GallonPerMinutePerAcre': eumUnit.eumUGallonPerMinutePerAcre,
    'GramPerGram': eumUnit.eumUGramPerGram,
    'GramPerKilogram': eumUnit.eumUGramPerKilogram,
    'GramPerMinute': eumUnit.eumUGramPerMinute,
    'HorsePower': eumUnit.eumUHorsePower,
    'HoursPerDay': eumUnit.eumUHoursPerDay,
    'InchPerDegreeFahrenheitPerDay': eumUnit.eumUInchPerDegreeFahrenheitPerDay,
    'InchPerDegreeFahrenheitPerHour': eumUnit.eumUInchPerDegreeFahrenheitPerHour,
    'Joule': eumUnit.eumUJoule,
    'JouleKilogramPerKelvin': eumUnit.eumUJouleKilogramPerKelvin,
    'JoulePerKilogram': eumUnit.eumUJoulePerKilogram,
    'JoulePerM2PerDay': eumUnit.eumUJoulePerM2PerDay,
    'KiloGramPerHectarPerDay': eumUnit.eumUKiloGramPerHectarPerDay,
    'KiloGramPerMeterPerSecond': eumUnit.eumUKiloGramPerMeterPerSecond,
    'KiloGramPerPersonPerDay': eumUnit.eumUKiloGramPerPersonPerDay,
    'KiloWattHour': eumUnit.eumUKiloWattHour,
    'KilogramPerKilogram': eumUnit.eumUKilogramPerKilogram,
    'KilogramPerMinute': eumUnit.eumUKilogramPerMinute,
    'KilogramPerS2': eumUnit.eumUKilogramPerS2,
    'LiterPerDayPerHead': eumUnit.eumULiterPerDayPerHead,
    'LiterPerDayPerMeter': eumUnit.eumULiterPerDayPerMeter,
    'LiterPerM2': eumUnit.eumULiterPerM2,
    'LiterPerM3': eumUnit.eumULiterPerM3,
    'LiterPerMinutePerHectar': eumUnit.eumULiterPerMinutePerHectar,
    'LiterPerMinutePerMeter': eumUnit.eumULiterPerMinutePerMeter,
    'LiterPerMinutePerSecond': eumUnit.eumULiterPerMinutePerSecond,
    'LiterPerSecondPerHectar': eumUnit.eumULiterPerSecondPerHectar,
    'LiterPerSecondPerMeter': eumUnit.eumULiterPerSecondPerMeter,
    'LiterPerSecondSquare': eumUnit.eumULiterPerSecondSquare,
    'LiterPergram': eumUnit.eumULiterPergram,
    'M3PerM3': eumUnit.eumUM3PerM3,
    'MegaGal': eumUnit.eumUMegaGal,
    'MegaLiter': eumUnit.eumUMegaLiter,
    'MegaLiterPerDayPerMeter': eumUnit.eumUMegaLiterPerDayPerMeter,
    'MegaPascal': eumUnit.eumUMegaPascal,
    'MegagalUK': eumUnit.eumUMegagalUK,
    'Meter2One3rdPerSec': eumUnit.eumUMeter2One3rdPerSec,
    'Meter2OneHalfPerSec': eumUnit.eumUMeter2OneHalfPerSec,
    'MeterPerMeter': eumUnit.eumUMeterPerMeter,
    'MeterPerSecondPerHectar': eumUnit.eumUMeterPerSecondPerHectar,
    'MeterPerSecondPerSecond': eumUnit.eumUMeterPerSecondPerSecond,
    'MetresOfWater': eumUnit.eumUMetresOfWater,
    'MgalPerDay': eumUnit.eumUMgalPerDay,
    'MgalPerDayPerPsi': eumUnit.eumUMgalPerDayPerPsi,
    'MgalUKPerDay': eumUnit.eumUMgalUKPerDay,
    'MgalUKPerDayPerPsi': eumUnit.eumUMgalUKPerDayPerPsi,
    'MicroMeterPerSecond': eumUnit.eumUMicroMeterPerSecond,
    'MicrogramPerGram': eumUnit.eumUMicrogramPerGram,
    'MicroliterPerLiter': eumUnit.eumUMicroliterPerLiter,
    'MilliAmpere': eumUnit.eumUMilliAmpere,
    'MilliGramPerLiterPerDay': eumUnit.eumUMilliGramPerLiterPerDay,
    'MilliMeterD50': eumUnit.eumUMilliMeterD50,
    'MilliMeterPerDegreeCelsiusPerDay': eumUnit.eumUMilliMeterPerDegreeCelsiusPerDay,
    'MilliMeterPerDegreeCelsiusPerHour': eumUnit.eumUMilliMeterPerDegreeCelsiusPerHour,
    'MilligramPerGram': eumUnit.eumUMilligramPerGram,
    'MilligramPerKilogram': eumUnit.eumUMilligramPerKilogram,
    'MilliliterPerLiter': eumUnit.eumUMilliliterPerLiter,
    'MillionGalPerDayPerAcre': eumUnit.eumUMillionGalPerDayPerAcre,
    'MillionGalPerDayPerSecond': eumUnit.eumUMillionGalPerDayPerSecond,
    'MillionGalUKPerDayPerAcre': eumUnit.eumUMillionGalUKPerDayPerAcre,
    'MillionGalUKPerDayPerSecond': eumUnit.eumUMillionGalUKPerDayPerSecond,
    'MillionLiterPerDayPerHectar': eumUnit.eumUMillionLiterPerDayPerHectar,
    'MillionLiterPerDayPerSecond': eumUnit.eumUMillionLiterPerDayPerSecond,
    'MlPerDay': eumUnit.eumUMlPerDay,
    'Newton': eumUnit.eumUNewton,
    'NewtonMeter': eumUnit.eumUNewtonMeter,
    'NewtonMeterSecond': eumUnit.eumUNewtonMeterSecond,
    'NewtonPerM3': eumUnit.eumUNewtonPerM3,
    'NewtonPerMeterPerSecond': eumUnit.eumUNewtonPerMeterPerSecond,
    'NewtonPerSqrMeter': eumUnit.eumUNewtonPerSqrMeter,
    'OnePerOne': eumUnit.eumUOnePerOne,
    'PSU': eumUnit.eumUPSU,
    'PSUM3PerSec': eumUnit.eumUPSUM3PerSec,
    'PSUft3PerSec': eumUnit.eumUPSUft3PerSec,
    'Pascal': eumUnit.eumUPascal,
    'PascalSecond': eumUnit.eumUPascalSecond,
    'Per100ml': eumUnit.eumUPer100ml,
    'PerAcre': eumUnit.eumUPerAcre,
    'PerCent': eumUnit.eumUPerCent,
    'PerCubicMeter': eumUnit.eumUPerCubicMeter,
    'PerDegreeCelsiusPerDay': eumUnit.eumUPerDegreeCelsiusPerDay,
    'PerDegreeCelsiusPerHour': eumUnit.eumUPerDegreeCelsiusPerHour,
    'PerDegreeFahrenheitPerDay': eumUnit.eumUPerDegreeFahrenheitPerDay,
    'PerDegreeFahrenheitPerHour': eumUnit.eumUPerDegreeFahrenheitPerHour,
    'PerHectar': eumUnit.eumUPerHectar,
    'PerM2PerSec': eumUnit.eumUPerM2PerSec,
    'PerMeterPerSecond': eumUnit.eumUPerMeterPerSecond,
    'PerMillion': eumUnit.eumUPerMillion,
    'PerSquareMeter': eumUnit.eumUPerSquareMeter,
    'PerThousand': eumUnit.eumUPerThousand,
    'PerWatt': eumUnit.eumUPerWatt,
    'Person': eumUnit.eumUPerson,
    'Pound': eumUnit.eumUPound,
    'PoundFeetPerSec': eumUnit.eumUPoundFeetPerSec,
    'PoundPerAcre': eumUnit.eumUPoundPerAcre,
    'PoundPerAcrePerDay': eumUnit.eumUPoundPerAcrePerDay,
    'PoundPerCubicFeet': eumUnit.eumUPoundPerCubicFeet,
    'PoundPerCubicFeetUS': eumUnit.eumUPoundPerCubicFeetUS,
    'PoundPerDay': eumUnit.eumUPoundPerDay,
    'PoundPerFeetPerSec2': eumUnit.eumUPoundPerFeetPerSec2,
    'PoundPerFt2PerSec': eumUnit.eumUPoundPerFt2PerSec,
    'PoundPerHa': eumUnit.eumUPoundPerHa,
    'PoundPerHour': eumUnit.eumUPoundPerHour,
    'PoundPerKm2': eumUnit.eumUPoundPerKm2,
    'PoundPerM2': eumUnit.eumUPoundPerM2,
    'PoundPerMeter': eumUnit.eumUPoundPerMeter,
    'PoundPerMi2': eumUnit.eumUPoundPerMi2,
    'PoundPerMinute': eumUnit.eumUPoundPerMinute,
    'PoundPerPersonPerDay': eumUnit.eumUPoundPerPersonPerDay,
    'PoundPerSecond': eumUnit.eumUPoundPerSecond,
    'PoundPerSquareFeet': eumUnit.eumUPoundPerSquareFeet,
    'PoundPerSquareFeetUS': eumUnit.eumUPoundPerSquareFeetUS,
    'PoundPerYear': eumUnit.eumUPoundPerYear,
    'PoundPeryard3': eumUnit.eumUPoundPeryard3,
    'PoundPerydUS3': eumUnit.eumUPoundPerydUS3,
    'PoundforceSecPerSqrFt': eumUnit.eumUPoundforceSecPerSqrFt,
    'RevolutionPerMinute': eumUnit.eumURevolutionPerMinute,
    'SecPerFeet2One3rd': eumUnit.eumUSecPerFeet2One3rd,
    'SecPerMeter': eumUnit.eumUSecPerMeter,
    'SecPerMeter2One3rd': eumUnit.eumUSecPerMeter2One3rd,
    'SquareFeetPerSecond': eumUnit.eumUSquareFeetPerSecond,
    'SquareFeetPerSecondPerAcre': eumUnit.eumUSquareFeetPerSecondPerAcre,
    'SquareMeterPerSecond': eumUnit.eumUSquareMeterPerSecond,
    'SquareMeterPerSecondPerHectar': eumUnit.eumUSquareMeterPerSecondPerHectar,
    'TenTo6m3': eumUnit.eumUTenTo6m3,
    'ThousandPerM2PerDay': eumUnit.eumUThousandPerM2PerDay,
    'TonPerDay': eumUnit.eumUTonPerDay,
    'TonPerSec': eumUnit.eumUTonPerSec,
    'TonPerYear': eumUnit.eumUTonPerYear,
    'UnitUndefined': eumUnit.eumUUnitUndefined,
    'WattPerM2': eumUnit.eumUWattPerM2,
    'WattPerMeter': eumUnit.eumUWattPerMeter,
    'WattSecond': eumUnit.eumUWattSecond,
    'Yard3': eumUnit.eumUYard3,
    'Yard3PerYard': eumUnit.eumUYard3PerYard,
    'acft': eumUnit.eumUacft,
    'acftPerDay': eumUnit.eumUacftPerDay,
    'acftPerDayPerPsi': eumUnit.eumUacftPerDayPerPsi,
    'acftPerSecPerAcre': eumUnit.eumUacftPerSecPerAcre,
    'acre': eumUnit.eumUacre,
    'acreUS': eumUnit.eumUacreUS,
    'ampere': eumUnit.eumUampere,
    'billionPerDay': eumUnit.eumUbillionPerDay,
    'centimeter': eumUnit.eumUcentimeter,
    'cm3PerSecPerCm': eumUnit.eumUcm3PerSecPerCm,
    'cmPerHour': eumUnit.eumUcmPerHour,
    'currencyPerYear': eumUnit.eumUcurrencyPerYear,
    'dB_re_1muPa2second': eumUnit.eumUdB_re_1muPa2second,
    'dBperLambda': eumUnit.eumUdBperLambda,
    'day': eumUnit.eumUday,
    'degree': eumUnit.eumUdegree,
    'degreeCelsius': eumUnit.eumUdegreeCelsius,
    'degreeFahrenheit': eumUnit.eumUdegreeFahrenheit,
    'degreeKelvin': eumUnit.eumUdegreeKelvin,
    'degreePerMeter': eumUnit.eumUdegreePerMeter,
    'degreePerSecond': eumUnit.eumUdegreePerSecond,
    'degreesquared': eumUnit.eumUdegreesquared,
    'deltaDegreeCelsius': eumUnit.eumUdeltaDegreeCelsius,
    'deltaDegreeFahrenheit': eumUnit.eumUdeltaDegreeFahrenheit,
    'exaJoule': eumUnit.eumUexaJoule,
    'feet': eumUnit.eumUfeet,
    'feetPerDay': eumUnit.eumUfeetPerDay,
    'feetPerHour': eumUnit.eumUfeetPerHour,
    'feetPerMinute': eumUnit.eumUfeetPerMinute,
    'feetPerSec': eumUnit.eumUfeetPerSec,
    'feetUS': eumUnit.eumUfeetUS,
    'feetUSPerDay': eumUnit.eumUfeetUSPerDay,
    'feetUSPerSecond': eumUnit.eumUfeetUSPerSecond,
    'ft2': eumUnit.eumUft2,
    'ft2PerDay': eumUnit.eumUft2PerDay,
    'ft2PerDegree': eumUnit.eumUft2PerDegree,
    'ft2PerHertz': eumUnit.eumUft2PerHertz,
    'ft2PerHertzPerDegree': eumUnit.eumUft2PerHertzPerDegree,
    'ft2PerHertzPerRadian': eumUnit.eumUft2PerHertzPerRadian,
    'ft2PerHour': eumUnit.eumUft2PerHour,
    'ft2PerRad': eumUnit.eumUft2PerRad,
    'ft2PerSec': eumUnit.eumUft2PerSec,
    'ft2PerSec2': eumUnit.eumUft2PerSec2,
    'ft2PerSec3': eumUnit.eumUft2PerSec3,
    'ft2PerSec3PerRad': eumUnit.eumUft2PerSec3PerRad,
    'ft2PerSecPerDegree': eumUnit.eumUft2PerSecPerDegree,
    'ft2PerSecPerRad': eumUnit.eumUft2PerSecPerRad,
    'ft2Sec': eumUnit.eumUft2Sec,
    'ft2Sec2PerDegree': eumUnit.eumUft2Sec2PerDegree,
    'ft2Sec2PerRad': eumUnit.eumUft2Sec2PerRad,
    'ft2SecPerDegree': eumUnit.eumUft2SecPerDegree,
    'ft2SecPerRad': eumUnit.eumUft2SecPerRad,
    'ft3': eumUnit.eumUft3,
    'ft3PerDay': eumUnit.eumUft3PerDay,
    'ft3PerDayPerHead': eumUnit.eumUft3PerDayPerHead,
    'ft3PerFt': eumUnit.eumUft3PerFt,
    'ft3PerMin': eumUnit.eumUft3PerMin,
    'ft3PerSec': eumUnit.eumUft3PerSec,
    'ft3PerSec2': eumUnit.eumUft3PerSec2,
    'ft3PerSecPerFt': eumUnit.eumUft3PerSecPerFt,
    'ft3PerSecPerInch': eumUnit.eumUft3PerSecPerInch,
    'ft3PerSecPerMi2': eumUnit.eumUft3PerSecPerMi2,
    'ft3PerSecPerPsi': eumUnit.eumUft3PerSecPerPsi,
    'ft3PerYear': eumUnit.eumUft3PerYear,
    'ftUS2': eumUnit.eumUftUS2,
    'ftUS3PerDay': eumUnit.eumUftUS3PerDay,
    'ftUS3PerMin': eumUnit.eumUftUS3PerMin,
    'ftUS3PerSec': eumUnit.eumUftUS3PerSec,
    'ftUS3PerSecPerFtUS': eumUnit.eumUftUS3PerSecPerFtUS,
    'ftUS3PerYear': eumUnit.eumUftUS3PerYear,
    'ftUS3PerftUS': eumUnit.eumUftUS3PerftUS,
    'gal': eumUnit.eumUgal,
    'galUK': eumUnit.eumUgalUK,
    'galUKPerDay': eumUnit.eumUgalUKPerDay,
    'gallonPerMinPerPsi': eumUnit.eumUgallonPerMinPerPsi,
    'gigaJoule': eumUnit.eumUgigaJoule,
    'gigaWattHour': eumUnit.eumUgigaWattHour,
    'gigaWattPerMeter': eumUnit.eumUgigaWattPerMeter,
    'gram': eumUnit.eumUgram,
    'gramPerAcre': eumUnit.eumUgramPerAcre,
    'gramPerDay': eumUnit.eumUgramPerDay,
    'gramPerHa': eumUnit.eumUgramPerHa,
    'gramPerKm2': eumUnit.eumUgramPerKm2,
    'gramPerL': eumUnit.eumUgramPerL,
    'gramPerM2': eumUnit.eumUgramPerM2,
    'gramPerM2PerDay': eumUnit.eumUgramPerM2PerDay,
    'gramPerM2PerSec': eumUnit.eumUgramPerM2PerSec,
    'gramPerM3': eumUnit.eumUgramPerM3,
    'gramPerM3PerDay': eumUnit.eumUgramPerM3PerDay,
    'gramPerM3PerHour': eumUnit.eumUgramPerM3PerHour,
    'gramPerM3PerSec': eumUnit.eumUgramPerM3PerSec,
    'gramPerMi2': eumUnit.eumUgramPerMi2,
    'gramPerSec': eumUnit.eumUgramPerSec,
    'gramPerSquareFeet': eumUnit.eumUgramPerSquareFeet,
    'gwatt': eumUnit.eumUgwatt,
    'ha': eumUnit.eumUha,
    'hectoPascal': eumUnit.eumUhectoPascal,
    'hertz': eumUnit.eumUhertz,
    'hour': eumUnit.eumUhour,
    'in3PerSecPerIn': eumUnit.eumUin3PerSecPerIn,
    'inPerDay': eumUnit.eumUinPerDay,
    'inUS3PerSecPerInUS': eumUnit.eumUinUS3PerSecPerInUS,
    'inch': eumUnit.eumUinch,
    'inchPerHour': eumUnit.eumUinchPerHour,
    'inchPerMinute': eumUnit.eumUinchPerMinute,
    'inchPerSec': eumUnit.eumUinchPerSec,
    'inchUS': eumUnit.eumUinchUS,
    'inchUSPerHour': eumUnit.eumUinchUSPerHour,
    'inchUSPerMinute': eumUnit.eumUinchUSPerMinute,
    'inchUSPerSecond': eumUnit.eumUinchUSPerSecond,
    'intCode': eumUnit.eumUintCode,
    'kiloGramPerM3': eumUnit.eumUkiloGramPerM3,
    'kiloJoule': eumUnit.eumUkiloJoule,
    'kiloJoulePerM2PerDay': eumUnit.eumUkiloJoulePerM2PerDay,
    'kiloJoulePerM2PerHour': eumUnit.eumUkiloJoulePerM2PerHour,
    'kiloNewton': eumUnit.eumUkiloNewton,
    'kiloNewtonPerM3': eumUnit.eumUkiloNewtonPerM3,
    'kiloNewtonPerSqrMeter': eumUnit.eumUkiloNewtonPerSqrMeter,
    'kiloPascal': eumUnit.eumUkiloPascal,
    'kiloWattPerFeet': eumUnit.eumUkiloWattPerFeet,
    'kiloWattPerMeter': eumUnit.eumUkiloWattPerMeter,
    'kilogram': eumUnit.eumUkilogram,
    'kilogramM2': eumUnit.eumUkilogramM2,
    'kilogramMeter': eumUnit.eumUkilogramMeter,
    'kilogramMeter2': eumUnit.eumUkilogramMeter2,
    'kilogramMeter2PerSecond': eumUnit.eumUkilogramMeter2PerSecond,
    'kilogramMeterPerSecond': eumUnit.eumUkilogramMeterPerSecond,
    'kilogramPerAcre': eumUnit.eumUkilogramPerAcre,
    'kilogramPerDay': eumUnit.eumUkilogramPerDay,
    'kilogramPerHa': eumUnit.eumUkilogramPerHa,
    'kilogramPerHaPerHour': eumUnit.eumUkilogramPerHaPerHour,
    'kilogramPerHour': eumUnit.eumUkilogramPerHour,
    'kilogramPerKm2': eumUnit.eumUkilogramPerKm2,
    'kilogramPerM': eumUnit.eumUkilogramPerM,
    'kilogramPerM2': eumUnit.eumUkilogramPerM2,
    'kilogramPerM2PerDay': eumUnit.eumUkilogramPerM2PerDay,
    'kilogramPerM2PerSec': eumUnit.eumUkilogramPerM2PerSec,
    'kilogramPerMi2': eumUnit.eumUkilogramPerMi2,
    'kilogramPerSec': eumUnit.eumUkilogramPerSec,
    'kilogramPerSecPerM': eumUnit.eumUkilogramPerSecPerM,
    'kilogramPerSquareFeet': eumUnit.eumUkilogramPerSquareFeet,
    'kilogramPerYear': eumUnit.eumUkilogramPerYear,
    'kilometer': eumUnit.eumUkilometer,
    'kilometerPerHour': eumUnit.eumUkilometerPerHour,
    'kiloton': eumUnit.eumUkiloton,
    'km2': eumUnit.eumUkm2,
    'km3': eumUnit.eumUkm3,
    'knot': eumUnit.eumUknot,
    'kwatt': eumUnit.eumUkwatt,
    'liter': eumUnit.eumUliter,
    'literPerMin': eumUnit.eumUliterPerMin,
    'literPerMinPerMeter': eumUnit.eumUliterPerMinPerMeter,
    'literPerPersonPerDay': eumUnit.eumUliterPerPersonPerDay,
    'literPerSec': eumUnit.eumUliterPerSec,
    'literPerSecPerKm2': eumUnit.eumUliterPerSecPerKm2,
    'literPerSecPerMeter': eumUnit.eumUliterPerSecPerMeter,
    'm2': eumUnit.eumUm2,
    'm2PerDay': eumUnit.eumUm2PerDay,
    'm2PerDegree': eumUnit.eumUm2PerDegree,
    'm2PerHertz': eumUnit.eumUm2PerHertz,
    'm2PerHertz2': eumUnit.eumUm2PerHertz2,
    'm2PerHertz2PerDegree': eumUnit.eumUm2PerHertz2PerDegree,
    'm2PerHertz2PerRadian': eumUnit.eumUm2PerHertz2PerRadian,
    'm2PerHertzPerDegree': eumUnit.eumUm2PerHertzPerDegree,
    'm2PerHertzPerRadian': eumUnit.eumUm2PerHertzPerRadian,
    'm2PerHour': eumUnit.eumUm2PerHour,
    'm2PerRad': eumUnit.eumUm2PerRad,
    'm2PerSec': eumUnit.eumUm2PerSec,
    'm2PerSec2': eumUnit.eumUm2PerSec2,
    'm2PerSec3': eumUnit.eumUm2PerSec3,
    'm2PerSec3PerRad': eumUnit.eumUm2PerSec3PerRad,
    'm2PerSecPerDegree': eumUnit.eumUm2PerSecPerDegree,
    'm2PerSecPerRad': eumUnit.eumUm2PerSecPerRad,
    'm2Perkilogram': eumUnit.eumUm2Perkilogram,
    'm2Sec': eumUnit.eumUm2Sec,
    'm2Sec2PerDegree': eumUnit.eumUm2Sec2PerDegree,
    'm2Sec2PerRad': eumUnit.eumUm2Sec2PerRad,
    'm2SecPerDegree': eumUnit.eumUm2SecPerDegree,
    'm2SecPerRad': eumUnit.eumUm2SecPerRad,
    'm2mmPerKiloJoule': eumUnit.eumUm2mmPerKiloJoule,
    'm2mmPerMegaJoule': eumUnit.eumUm2mmPerMegaJoule,
    'm3': eumUnit.eumUm3,
    'm3PerCurrency': eumUnit.eumUm3PerCurrency,
    'm3PerDay': eumUnit.eumUm3PerDay,
    'm3PerDayPerHead': eumUnit.eumUm3PerDayPerHead,
    'm3PerDayPerMeter': eumUnit.eumUm3PerDayPerMeter,
    'm3PerHour': eumUnit.eumUm3PerHour,
    'm3PerHourPerMeter': eumUnit.eumUm3PerHourPerMeter,
    'm3PerKm2PerDay': eumUnit.eumUm3PerKm2PerDay,
    'm3PerM': eumUnit.eumUm3PerM,
    'm3PerMicrogram': eumUnit.eumUm3PerMicrogram,
    'm3PerMilligram': eumUnit.eumUm3PerMilligram,
    'm3PerMinute': eumUnit.eumUm3PerMinute,
    'm3PerS2': eumUnit.eumUm3PerS2,
    'm3PerSec': eumUnit.eumUm3PerSec,
    'm3PerSec2': eumUnit.eumUm3PerSec2,
    'm3PerSecPer10mm': eumUnit.eumUm3PerSecPer10mm,
    'm3PerSecPerHead': eumUnit.eumUm3PerSecPerHead,
    'm3PerSecPerM': eumUnit.eumUm3PerSecPerM,
    'm3PerYear': eumUnit.eumUm3PerYear,
    'm3PerYearPerM': eumUnit.eumUm3PerYearPerM,
    'm3Pergram': eumUnit.eumUm3Pergram,
    'megaJoule': eumUnit.eumUmegaJoule,
    'megaJoulePerM2PerDay': eumUnit.eumUmegaJoulePerM2PerDay,
    'megaNewton': eumUnit.eumUmegaNewton,
    'megaWattHour': eumUnit.eumUmegaWattHour,
    'megaWattPerMeter': eumUnit.eumUmegaWattPerMeter,
    'megaton': eumUnit.eumUmegaton,
    'meter': eumUnit.eumUmeter,
    'meterPerDay': eumUnit.eumUmeterPerDay,
    'meterPerHour': eumUnit.eumUmeterPerHour,
    'meterPerMinute': eumUnit.eumUmeterPerMinute,
    'meterPerSec': eumUnit.eumUmeterPerSec,
    'mgal': eumUnit.eumUmgal,
    'mi2': eumUnit.eumUmi2,
    'miUS2': eumUnit.eumUmiUS2,
    'microGramPerL': eumUnit.eumUmicroGramPerL,
    'microGramPerM2': eumUnit.eumUmicroGramPerM2,
    'microGramPerM3': eumUnit.eumUmicroGramPerM3,
    'microPascal': eumUnit.eumUmicroPascal,
    'microSiemens': eumUnit.eumUmicroSiemens,
    'microSiemensPerCentimeter': eumUnit.eumUmicroSiemensPerCentimeter,
    'microgram': eumUnit.eumUmicrogram,
    'microgramPerSec': eumUnit.eumUmicrogramPerSec,
    'micrometer': eumUnit.eumUmicrometer,
    'mile': eumUnit.eumUmile,
    'milePerHour': eumUnit.eumUmilePerHour,
    'mileUS': eumUnit.eumUmileUS,
    'millPer100ml': eumUnit.eumUmillPer100ml,
    'milliBar': eumUnit.eumUmilliBar,
    'milliGramPerL': eumUnit.eumUmilliGramPerL,
    'milliGramPerL2OneHalfPerDay': eumUnit.eumUmilliGramPerL2OneHalfPerDay,
    'milliGramPerL2OneHalfPerHour': eumUnit.eumUmilliGramPerL2OneHalfPerHour,
    'milliGramPerM3': eumUnit.eumUmilliGramPerM3,
    'milliSiemens': eumUnit.eumUmilliSiemens,
    'milliSiemensPerCentimeter': eumUnit.eumUmilliSiemensPerCentimeter,
    'milliVolt': eumUnit.eumUmilliVolt,
    'millifeet': eumUnit.eumUmillifeet,
    'milligram': eumUnit.eumUmilligram,
    'milligramPerAcre': eumUnit.eumUmilligramPerAcre,
    'milligramPerHa': eumUnit.eumUmilligramPerHa,
    'milligramPerKm2': eumUnit.eumUmilligramPerKm2,
    'milligramPerM2': eumUnit.eumUmilligramPerM2,
    'milligramPerMi2': eumUnit.eumUmilligramPerMi2,
    'milligramPerSec': eumUnit.eumUmilligramPerSec,
    'milligramPerSquareFeet': eumUnit.eumUmilligramPerSquareFeet,
    'milliliter': eumUnit.eumUmilliliter,
    'millimeter': eumUnit.eumUmillimeter,
    'millimeterPerDay': eumUnit.eumUmillimeterPerDay,
    'millimeterPerHour': eumUnit.eumUmillimeterPerHour,
    'millimeterPerMonth': eumUnit.eumUmillimeterPerMonth,
    'millimeterPerSecond': eumUnit.eumUmillimeterPerSecond,
    'millimeterPerYear': eumUnit.eumUmillimeterPerYear,
    'minute': eumUnit.eumUminute,
    'mm3PerSecPerMm': eumUnit.eumUmm3PerSecPerMm,
    'mmPerDay': eumUnit.eumUmmPerDay,
    'month': eumUnit.eumUmonth,
    'mwatt': eumUnit.eumUmwatt,
    'nauticalmile': eumUnit.eumUnauticalmile,
    'perAcft': eumUnit.eumUperAcft,
    'perDay': eumUnit.eumUperDay,
    'perDegreeCelsius': eumUnit.eumUperDegreeCelsius,
    'perDegreeFahrenheit': eumUnit.eumUperDegreeFahrenheit,
    'perExaJoule': eumUnit.eumUperExaJoule,
    'perFeet': eumUnit.eumUperFeet,
    'perFeetUS': eumUnit.eumUperFeetUS,
    'perFt3': eumUnit.eumUperFt3,
    'perGallon': eumUnit.eumUperGallon,
    'perGallonUK': eumUnit.eumUperGallonUK,
    'perGigaJoule': eumUnit.eumUperGigaJoule,
    'perGigaWattHour': eumUnit.eumUperGigaWattHour,
    'perGram': eumUnit.eumUperGram,
    'perHour': eumUnit.eumUperHour,
    'perInch': eumUnit.eumUperInch,
    'perInchUS': eumUnit.eumUperInchUS,
    'perJoule': eumUnit.eumUperJoule,
    'perKiloJoule': eumUnit.eumUperKiloJoule,
    'perKiloWattHour': eumUnit.eumUperKiloWattHour,
    'perKilogram': eumUnit.eumUperKilogram,
    'perKiloton': eumUnit.eumUperKiloton,
    'perKm2': eumUnit.eumUperKm2,
    'perKm3': eumUnit.eumUperKm3,
    'perLiter': eumUnit.eumUperLiter,
    'perM3': eumUnit.eumUperM3,
    'perMegaJoule': eumUnit.eumUperMegaJoule,
    'perMegaWattHour': eumUnit.eumUperMegaWattHour,
    'perMegagallon': eumUnit.eumUperMegagallon,
    'perMegagallonUK': eumUnit.eumUperMegagallonUK,
    'perMegaliter': eumUnit.eumUperMegaliter,
    'perMegaton': eumUnit.eumUperMegaton,
    'perMeter': eumUnit.eumUperMeter,
    'perMicrogram': eumUnit.eumUperMicrogram,
    'perMilligallon': eumUnit.eumUperMilligallon,
    'perMilligram': eumUnit.eumUperMilligram,
    'perMilliliter': eumUnit.eumUperMilliliter,
    'perPetaJoule': eumUnit.eumUperPetaJoule,
    'perPound': eumUnit.eumUperPound,
    'perSec': eumUnit.eumUperSec,
    'perTeraJoule': eumUnit.eumUperTeraJoule,
    'perTon': eumUnit.eumUperTon,
    'perTonUS': eumUnit.eumUperTonUS,
    'perWattSecond': eumUnit.eumUperWattSecond,
    'perYard3': eumUnit.eumUperYard3,
    'perYardUS3': eumUnit.eumUperYardUS3,
    'percentPer100feet': eumUnit.eumUpercentPer100feet,
    'percentPer100meter': eumUnit.eumUpercentPer100meter,
    'percentPerDay': eumUnit.eumUpercentPerDay,
    'percentPerHour': eumUnit.eumUpercentPerHour,
    'percentPerMinute': eumUnit.eumUpercentPerMinute,
    'percentPerSecond': eumUnit.eumUpercentPerSecond,
    'perminute': eumUnit.eumUperminute,
    'permonth': eumUnit.eumUpermonth,
    'peryear': eumUnit.eumUperyear,
    'petaJoule': eumUnit.eumUpetaJoule,
    'psi': eumUnit.eumUpsi,
    'radian': eumUnit.eumUradian,
    'radianPerMeter': eumUnit.eumUradianPerMeter,
    'radianPerSecond': eumUnit.eumUradianPerSecond,
    'sec': eumUnit.eumUsec,
    'siemens': eumUnit.eumUsiemens,
    'siemensPerMeter': eumUnit.eumUsiemensPerMeter,
    'teraJoule': eumUnit.eumUteraJoule,
    'ton': eumUnit.eumUton,
    'tonPerAcre': eumUnit.eumUtonPerAcre,
    'tonPerHa': eumUnit.eumUtonPerHa,
    'tonPerKm2': eumUnit.eumUtonPerKm2,
    'tonPerM2': eumUnit.eumUtonPerM2,
    'tonPerM3': eumUnit.eumUtonPerM3,
    'tonPerMeter': eumUnit.eumUtonPerMeter,
    'tonPerMi2': eumUnit.eumUtonPerMi2,
    'tonPerSquareFeet': eumUnit.eumUtonPerSquareFeet,
    'tonUS': eumUnit.eumUtonUS,
    'trillionPerYear': eumUnit.eumUtrillionPerYear,
    'volt': eumUnit.eumUvolt,
    'watt': eumUnit.eumUwatt,
    'yard': eumUnit.eumUyard,
    'yard3PerSec': eumUnit.eumUyard3PerSec,
    'yard3PerSecPeryard': eumUnit.eumUyard3PerSecPeryard,
    'yard3PerYearPeryard': eumUnit.eumUyard3PerYearPeryard,
    'yardUS': eumUnit.eumUyardUS,
    'yardUS3PerSec': eumUnit.eumUyardUS3PerSec,
    'ydUS2': eumUnit.eumUydUS2,
    'ydUS3': eumUnit.eumUydUS3,
    'ydUS3PerSec': eumUnit.eumUydUS3PerSec,
    'ydUS3PerSecPerydUS': eumUnit.eumUydUS3PerSecPerydUS,
    'ydUS3PerYearPerydUS': eumUnit.eumUydUS3PerYearPerydUS,
    'ydUS3PeryardUS': eumUnit.eumUydUS3PeryardUS,
    'year': eumUnit.eumUyear
}
